@@ -4,31 +4,31 @@
  * Gnome Chemistry Utils
  * crystalatom.cc 
  *
- * Copyright (C) 2002-2005
+ * Copyright (C) 2002-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
- * Developed by Jean Brfort <jean.brefort@normalesup.org>
+ * This program is free software; you can redistribute it and/or 
+ * modify it under the terms of the GNU General Public License as 
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the 
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA  02111-1307, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
+ * USA
  */
 
+#include "config.h"
+#include "element.h"
 #include "crystalatom.h"
 #include "element.h"
 #include "xml-utils.h"
-#include <math.h>
 #include <GL/glu.h>
+#include <cmath>
 #include <string>
 
 using namespace gcu;
@@ -40,7 +40,8 @@ CrystalAtom::CrystalAtom(): Atom()
 	m_Radius.scale = NULL;
 	m_Radius.spin = GCU_N_A_SPIN,
 	m_Radius.charge = 0;
-	m_Radius.value = 0.0;
+	m_Radius.value.value = 0.0;
+	m_Radius.value.prec = 0;
 	m_Radius.cn = -1;
 	m_bCustomColor = false;
 	m_fRed = m_fBlue = m_fGreen = 0;
@@ -48,9 +49,8 @@ CrystalAtom::CrystalAtom(): Atom()
 	m_nCleave = 0;
 }
 
-CrystalAtom::~CrystalAtom()
+CrystalAtom::~CrystalAtom ()
 {
-	if (m_Radius.scale) g_free(m_Radius.scale);
 }
 
 CrystalAtom::CrystalAtom(int Z, double x, double y, double z): Atom(Z, x, y, z)
@@ -60,7 +60,8 @@ CrystalAtom::CrystalAtom(int Z, double x, double y, double z): Atom(Z, x, y, z)
 	m_Radius.scale = NULL;
 	m_Radius.spin = GCU_N_A_SPIN,
 	m_Radius.charge = 0;
-	m_Radius.value = 0.0;
+	m_Radius.value.value = 0.0;
+	m_Radius.value.prec = 0;
 	m_Radius.cn = -1;
 	m_fAlpha = 1.;
 	if (Z) SetDefaultColor();
@@ -105,7 +106,7 @@ void CrystalAtom::Draw()
 	quadObj = gluNewQuadric() ;
     gluQuadricDrawStyle(quadObj, GL_FILL);
 	gluQuadricNormals(quadObj, GL_SMOOTH) ;
-	gluSphere(quadObj, m_Radius.value, 20, 10) ;
+	gluSphere(quadObj, m_Radius.value.value, 20, 10) ;
 	gluDeleteQuadric(quadObj) ;
 	glPopMatrix() ;
 }
@@ -143,14 +144,17 @@ void CrystalAtom::SetSize(double r)
 	m_Radius.scale = NULL;
 	m_Radius.spin = GCU_N_A_SPIN,
 	m_Radius.charge = 0;
+	m_Radius.value.value = 0.0;
+	m_Radius.value.prec = 0;
 	m_Radius.cn = -1;
-	m_Radius.value = r;
+	m_Radius.value.value = 0.0;
+	m_Radius.value.prec = 0;
 }
 
 
 double CrystalAtom::GetSize()
 {
-	return m_Radius.value;
+	return m_Radius.value.value;
 }
 
 bool CrystalAtom::operator==(CrystalAtom& caAtom)
@@ -171,7 +175,7 @@ double CrystalAtom::Distance(double dx, double dy, double dz, bool bFixed)
 	dx -= x() ;
 	dy -= y() ;
 	dz -= z() ;
-	return sqrt(dx * dx + dy * dy + dz * dz) + m_Radius.value ;
+	return sqrt(dx * dx + dy * dy + dz * dz) + m_Radius.value.value ;
 }
 
 void CrystalAtom::NetToCartesian(double a, double b, double c, double alpha, double beta, double gamma)
@@ -214,9 +218,7 @@ void CrystalAtom::SetRadius(const GcuAtomicRadius& r)
 	m_Radius.type = r.type;
 	m_Radius.value = r.value;
 	m_Radius.charge = r.charge;
-	if (m_Radius.scale) g_free(m_Radius.scale);
-	if (r.scale) m_Radius.scale = g_strdup(r.scale);
-	else m_Radius.scale = NULL;
+	m_Radius.scale = r.scale;
 	m_Radius.cn = r.cn;	//coordination number: -1: unspecified
 	m_Radius.spin = r.spin;
 }
