@@ -4,24 +4,22 @@
  * Gnome Chemistry Utils
  * object.h 
  *
- * Copyright (C) 2002-2004
+ * Copyright (C) 2002-2005 Jean Bréfort <jean.brefort@normalesup.org>
  *
- * Developed by Jean Bréfort <jean.brefort@normalesup.rg>
+ * This program is free software; you can redistribute it and/or 
+ * modify it under the terms of the GNU General Public License as 
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the 
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA  02111-1307, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
+ * USA
  */
 
 #ifndef GCU_OBJECT_H
@@ -426,9 +424,9 @@ It should not be called by a program; call Object::EmitSignal instead.
 	virtual bool OnSignal (SignalId Signal, Object *Child);
 
 /*!
-@param state: whether to bloc signals or not
+@param state: whether to block signals or not
 
-Blocks signals if state is true and unblocks if state is false.
+Blocks signals if State is true and unblocs if state is false.
 
 Since 0.4.2
 */
@@ -436,8 +434,6 @@ Since 0.4.2
 
 /*!
 
-/*!
-
 @return true if signals are locked, false otherwise
 
 Since 0.4.2
