@@ -4,9 +4,7 @@
  * Gnome Chemistry Utils
  * programs/gchemcalc.cc 
  *
- * Copyright (C) 2005
- *
- * Developed by Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2005-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -25,9 +23,6 @@
  */
 
 #include "config.h"
-#warning "the following lines should be removed for stable releases"
-#undef PACKAGE
-#define PACKAGE "gchemutils-unstable" 
 #include <gcu/element.h>
 #include <gcu/formula.h>
 #include <glib/gi18n.h>
@@ -42,24 +37,35 @@
 #include <gtk/gtkbox.h>
 #include <gtk/gtktreeview.h>
 #include <gtk/gtkcellrenderertext.h>
+#include <gtk/gtkclipboard.h>
 #include <goffice/goffice.h>
 #include <goffice/app/go-plugin.h>
 #include <goffice/app/go-plugin-loader-module.h>
 #include <goffice/data/go-data-simple.h>
 #include <goffice/gtk/go-graph-widget.h>
+#include <goffice/gtk/goffice-gtk.h>
 #include <goffice/graph/gog-axis.h>
 #include <goffice/graph/gog-data-set.h>
 #include <goffice/graph/gog-object.h>
 #include <goffice/graph/gog-plot.h>
+#include <goffice/graph/gog-renderer-pixbuf.h>
+#include <goffice/graph/gog-renderer-svg.h>
 #include <goffice/graph/gog-series.h>
 #include <goffice/graph/gog-style.h>
 #include <goffice/graph/gog-styled-object.h>
+#include <goffice/utils/go-format.h>
 #include <goffice/utils/go-line.h>
 #include <goffice/utils/go-marker.h>
-#include <math.h>
+#include <gsf/gsf-input-memory.h>
+#include <gsf/gsf-output-memory.h>
+#include <libgnomevfs/gnome-vfs.h>
+#include <cmath>
+#include <iostream>
 
 using namespace gcu;
 
+using namespace std;
+
 class GChemCalc {
 public:
 	GChemCalc ();
@@ -84,13 +90,24 @@ static void on_quit (GtkWidget *widget, void *data)
 	gtk_main_quit();
 }
 
+static void on_about_activate_url (GtkAboutDialog *about, const gchar *url, gpointer data)
+{
+	GnomeVFSResult error = gnome_vfs_url_show(url);
+	if (error != GNOME_VFS_OK) {
+		g_print("GnomeVFSResult while trying to launch URL in about dialog: error %u\n", error);
+	}
+}
+
 static void on_about (GtkWidget *widget, void *data)
 {
-	char * authors[] = {"Jean Bréfort", NULL};
-//	char * documentors[] = {NULL};
-	char license[] = "This program is free software; you can redistribute it and/or\n" 
+	const gchar * authors[] = {"Jean Bréfort", NULL};
+	const gchar * comments = _("GChemCalc is a simple calculator for chemists");
+	/* const gchar * documentors[] = {NULL}; */
+	const gchar * copyright = "Copyright \xc2\xa9 2005,2006 Jean Bréfort\n";
+	const gchar * license =
+		"This program is free software; you can redistribute it and/or\n"
 		"modify it under the terms of the GNU General Public License as\n"
- 		"published by the Free Software Foundation; either version 2 of the\n"
+		"published by the Free Software Foundation; either version 2 of the\n"
 		"License, or (at your option) any later version.\n\n"
 		"This program is distributed in the hope that it will be useful,\n"
 		"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
@@ -100,19 +117,21 @@ static void on_about (GtkWidget *widget, void *data)
 		"along with this program; if not, write to the Free Software\n"
 		"Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02111-1307\n"
 		"USA";
-/* Note to translators: replace the following string with the appropriate credits for you lang */
-	char *translator_credits = _("translator_credits");
+	
+	gtk_about_dialog_set_url_hook(on_about_activate_url, NULL, NULL);
+
+	/* Note to translators: replace the following string with the appropriate credits for you lang */
+	const gchar * translator_credits = _("translator_credits");
 	gtk_show_about_dialog (NULL,
-					"name", "GChemCalc",
-					"authors", authors,
-					"comments", _("GChemCalc is a simple calculator for chemists"),
-					"copyright", _("(C) 2005 by Jean Bréfort"),
-					"license", license,
-					"translator_credits", strcmp (translator_credits, "translator_credits") != 0 ? 
-											(const char *)translator_credits : NULL,
-					"version", VERSION,
-					"website", "http://www.nongnu.org/gchemutils",
-					NULL);
+	                       "name", "GChemCalc",
+	                       "authors", authors,
+	                       "comments", comments,
+	                       "copyright", copyright,
+	                       "license", license,
+	                       "translator_credits", translator_credits,
+	                       "version", VERSION,
+	                       "website", "http://www.nongnu.org/gchemutils",
+	                       NULL);
 }
 
 static void cb_entry_active (GtkEntry *entry, gpointer data)
@@ -279,12 +298,116 @@ static void cb_entry_active (GtkEntry *entry, gpointer data)
 	}
 }
 
+static gboolean gsf_gdk_pixbuf_save (const char *buf,
+			 gsize count,
+			 GError **error,
+			 gpointer data)
+{
+	GsfOutput *output = GSF_OUTPUT (data);
+	gboolean ok = gsf_output_write (output, count, (const guint8*) buf);
+
+	if (!ok && error)
+		*error = g_error_copy (gsf_output_error (output));
+
+	return ok;
+}
+
+static void on_get_data (GtkClipboard *clipboard, GtkSelectionData *selection_data,  guint info, GogGraph *graph)
+{
+	guchar *buffer = NULL;
+	char *format = NULL;
+	GsfOutput *output;
+	GsfOutputMemory *omem;
+	gsf_off_t osize;
+	GError *error = NULL;
+	double w, h;
+	gog_graph_get_size (graph, &w, &h);
+	output = gsf_output_memory_new ();
+	omem   = GSF_OUTPUT_MEMORY (output);
+	switch (info) {
+	case 0: {
+			GsfXMLOut *xout;
+			char *old_num_locale, *old_monetary_locale;
+		
+			old_num_locale = g_strdup (go_setlocale (LC_NUMERIC, NULL));
+			go_setlocale (LC_NUMERIC, "C");
+			old_monetary_locale = g_strdup (go_setlocale (LC_MONETARY, NULL));
+			go_setlocale (LC_MONETARY, "C");
+			go_set_untranslated_bools ();
+		
+			xout = gsf_xml_out_new (output);
+			gog_object_write_xml_sax (GOG_OBJECT (graph), xout);
+			g_object_unref (xout);
+		
+			/* go_setlocale restores bools to locale translation */
+			go_setlocale (LC_MONETARY, old_monetary_locale);
+			g_free (old_monetary_locale);
+			go_setlocale (LC_NUMERIC, old_num_locale);
+			g_free (old_num_locale);
+		}
+		break;
+	case 1:
+	case 2:
+		gog_graph_export_to_svg (graph, output, w, h, 1.0);
+		break;
+	default: {
+			GogRendererPixbuf *prend = GOG_RENDERER_PIXBUF (
+				g_object_new (GOG_RENDERER_PIXBUF_TYPE,
+						  "model", graph,
+						  NULL));
+			GdkPixbuf *pixbuf = gog_renderer_pixbuf_get (prend);
+	
+			if (!pixbuf) {
+				gog_renderer_pixbuf_update (prend, (int) w, (int) h, 1.);
+				pixbuf = gog_renderer_pixbuf_get (prend);
+			}
+			gdk_pixbuf_save_to_callback (pixbuf,
+							   gsf_gdk_pixbuf_save,
+							   output, format,
+							   &error, NULL);
+			g_object_unref (prend);
+		}
+		break;
+	}
+	osize = gsf_output_size (output);
+			
+	buffer = (guchar*) g_malloc (osize);
+	memcpy (buffer, gsf_output_memory_get_bytes (omem), osize);
+	gsf_output_close (output);
+	g_object_unref (output);
+	g_free (format);
+	gtk_selection_data_set (selection_data,
+				selection_data->target, 8,
+				(guchar *) buffer, osize);
+	g_free (buffer);
+}
+
+void on_clear_data(GtkClipboard *clipboard, GogGraph *graph)
+{
+	g_object_unref (graph);
+}
+
+static GtkTargetEntry const targets[] = {
+	{(char *) "application/x-goffice-graph",  0, 0},
+	{(char *) "image/svg+xml", 0, 2},
+	{(char *) "image/svg", 0, 1},
+	{(char *) "image/png", 0, 3}
+};
+
+static void on_copy (GOGraphWidget *widget)
+{
+	GtkClipboard* clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
+	gtk_clipboard_set_with_data (clipboard, targets, 4,
+		(GtkClipboardGetFunc) on_get_data, (GtkClipboardClearFunc) on_clear_data,
+		gog_object_dup (GOG_OBJECT (go_graph_widget_get_graph (widget)), NULL, NULL));
+}
+
 static GtkActionEntry entries[] = {
   { "FileMenu", NULL, N_("_File") },
 	  { "Quit", GTK_STOCK_QUIT, N_("_Quit"), "<control>Q",
 		  N_("Quit GChemCalc"), G_CALLBACK (on_quit) },
   { "HelpMenu", NULL, N_("_Help") },
-	  { "About", NULL, N_("_About"), NULL,
+	  { "About", GTK_STOCK_ABOUT, N_("_About"), NULL,
 		  N_("About GChemCalc"), G_CALLBACK (on_about) }
 };
 
@@ -300,20 +423,55 @@ static const char *ui_description =
 "  </menubar>"
 "</ui>";
 
+gboolean cb_print_version (const gchar *option_name, const gchar *value, gpointer data, GError **error)
+{
+	char *version = g_strconcat (_("GChemCalc Calculator version: "), VERSION, NULL);
+	puts (version);
+	g_free (version);
+	exit (0);
+	return TRUE;
+}
+
+static GOptionEntry options[] = 
+{
+  { "version", 'v', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, (void*) cb_print_version, "prints GChemCalc version", NULL },
+   { NULL }
+};
+
 int main (int argc, char *argv[])
 {
-	bindtextdomain (GETTEXT_PACKAGE, DATADIR"/locale");
-#ifdef ENABLE_NLS
-	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-#endif
+	GOptionContext *context;
+	GError *error = NULL;
 	textdomain (GETTEXT_PACKAGE);
+
 	gtk_init (&argc, &argv);
+	if (argc > 1 && argv[1][0] == '-') {
+		context = g_option_context_new (_(" [formula]"));
+		g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
+		g_option_context_add_group (context, gtk_get_option_group (TRUE));
+		g_option_context_set_help_enabled (context, TRUE);
+		g_option_context_parse (context, &argc, &argv, &error);
+		if (error) {
+			puts (error->message);
+			g_error_free (error);
+			return -1;
+		}
+	} else {
+		argc --;
+		argv ++;
+	}
+
+	if (argc > 1) {
+		cout << _("For usage see: gchemcalc [-?|--help]") << endl;
+		return -1;
+	}
+	
 	/* Initialize libgoffice */
 	libgoffice_init ();
 	/* Initialize plugins manager */
 	go_plugins_init (NULL, NULL, NULL, NULL, TRUE, GO_PLUGIN_LOADER_MODULE_TYPE);
 
-	GladeXML *xml =  glade_xml_new (DATADIR"/"PACKAGE"/glade/gchemcalc.glade", "gchemcalc", NULL);
+	GladeXML *xml =  glade_xml_new (GLADEDIR"/gchemcalc.glade", "gchemcalc", NULL);
 	GtkWidget *window = glade_xml_get_widget (xml, "gchemcalc");
 	g_signal_connect (GTK_OBJECT (window), "destroy",
 		 G_CALLBACK (gtk_main_quit),
@@ -327,7 +485,6 @@ int main (int argc, char *argv[])
 	gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
 	GtkAccelGroup *accel_group = gtk_ui_manager_get_accel_group (ui_manager);
 	gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
-	GError *error = NULL;
 	if (!gtk_ui_manager_add_ui_from_string (ui_manager, ui_description, -1, &error)) {
 		g_message ("building menus failed: %s", error->message);
 		g_error_free (error);
@@ -344,6 +501,7 @@ int main (int argc, char *argv[])
 	App.pclist = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
 	GtkTreeView *tree = GTK_TREE_VIEW (glade_xml_get_widget (xml, "composition"));
 	gtk_tree_view_set_model (tree, GTK_TREE_MODEL (App.pclist));
+	g_object_unref (App.pclist);
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *column;
 	/* column for element */
@@ -368,20 +526,20 @@ int main (int argc, char *argv[])
 	App.monomass = GTK_LABEL (glade_xml_get_widget (xml, "monomass"));
 	App.pattern_page = glade_xml_get_widget (xml, "pattern");
 #ifdef GO_GRAPH_WIDGET_OLD_API
-	GtkWidget *w = go_graph_widget_new ();
+	GtkWidget *pw = go_graph_widget_new ();
 #else
-	GtkWidget *w = go_graph_widget_new (NULL);
+	GtkWidget *pw = go_graph_widget_new (NULL);
 #endif
-	gtk_widget_show (w);
-	gtk_box_pack_end (GTK_BOX (App.pattern_page), w, TRUE, TRUE, 0);
-	App.chart = go_graph_widget_get_chart (GO_GRAPH_WIDGET (w));
+	gtk_widget_show (pw);
+	gtk_box_pack_end (GTK_BOX (App.pattern_page), pw, TRUE, TRUE, 0);
+	App.chart = go_graph_widget_get_chart (GO_GRAPH_WIDGET (pw));
 	App.plot = (GogPlot *) gog_plot_new_by_name ("GogXYPlot");
 	gog_object_add_by_name (GOG_OBJECT (App.chart), "Plot", GOG_OBJECT (App.plot));
 	// Create a series for the plot and populate it with some simple data
 	App.series = gog_plot_new_series (App.plot);
 	gog_object_add_by_name (GOG_OBJECT (App.series), "Vertical drop lines", NULL);
 	GogStyle *style = gog_styled_object_get_style (GOG_STYLED_OBJECT (App.series));
-	style->marker.mark->shape = GO_MARKER_NONE;
+	go_marker_set_shape (style->marker.mark, GO_MARKER_NONE);
 	style->marker.auto_shape = false;
 	style->line.dash_type = GO_LINE_NONE;
 	style->line.auto_dash = false;
@@ -391,11 +549,19 @@ int main (int argc, char *argv[])
 	gog_dataset_set_dim (GOG_DATASET (obj), GOG_AXIS_ELEM_MAX, data, &error);
 
 	gtk_widget_hide (App.pattern_page);
-	w = glade_xml_get_widget (xml, "entry");
+	GtkWidget *w = glade_xml_get_widget (xml, "entry");
 	g_signal_connect (GTK_OBJECT (w), "activate",
 		 G_CALLBACK (cb_entry_active),
 		 window);
-	gcu_element_load_databases ("isotopes", NULL);
+	gcu_element_load_databases (const_cast<char*> ("isotopes"), NULL);
+	w = glade_xml_get_widget (xml, "copy");
+	g_signal_connect_swapped (w, "clicked", G_CALLBACK (on_copy), pw);
+
+	if (argc == 1){
+		gtk_entry_set_text (GTK_ENTRY (w), argv[0]);
+		cb_entry_active (GTK_ENTRY (w), window);
+	}
+
 	gtk_main ();
 	return 0;
 }
